SystemOrganization addCategory: #'Path-Finder-CallTree'!
SystemOrganization addCategory: #'Path-Finder-Debugger'!
SystemOrganization addCategory: #'Path-Finder-TestChooser'!
SystemOrganization addCategory: #'Path-Finder-Tracer'!

SCodePaneMorph subclass: #SDebuggerCodePaneMorph
	instanceVariableNames: 'suspiciousColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Debugger'!

!SDebuggerCodePaneMorph commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SDebuggerCodePaneMorph class methodsFor: 'instance creation' stamp: 'mp 8/13/2013 13:49:51.143'!
on: aCallNode color: aColor
	"Label morph will be created later on"
	^ self new
		callNode: aCallNode;
		color: aColor;
		createDefinitionMorph;
		createNodeButtons;
		createObjectExploreButtons;
		yourself! !

!SDebuggerCodePaneMorph class methodsFor: 'instance creation' stamp: 'mp 3/29/2012 17:09:50.336'!
on: aCallNode color: aColor suspiciousColor: anotherColor

	^ (self on: aCallNode color: aColor)
		suspiciousColor: anotherColor;
		createLabelMorph;
		createExclamationMark;
		yourself
! !

!SDebuggerCodePaneMorph methodsFor: 'buttons' stamp: 'mp 8/13/2013 09:59:20.696'!
buttonSpec
	| spec |
	spec := super buttonSpec.
	spec add: {#faultLocalizationForMethod. UiFugueIcons magnifierExclamationIcon. 'Localize faults at statements'} afterIndex: 3.
	^ spec! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 9/21/2011 12:19:44.05'!
computeOchiaiFor: aDictionary
	| allByteCodes |
	allByteCodes := aDictionary values 
						inject: Set new into: [:set :each | set addAll: each; yourself].
	^ (allByteCodes collect: [:each | self ochiaiForByteCode: each from: aDictionary]) 
		sorted: [:a :b | a value first < b value first]! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 9/21/2011 12:08:41.012'!
displayTest: aTest from: aNumber

	^ 'Processing ', aTest asString, ' out of ', aNumber asString, ' Entry Points.'! !

!SDebuggerCodePaneMorph methodsFor: 'button actions' stamp: 'mp 9/21/2011 12:06:20.914'!
faultLocalizationForMethod
	| byteCodes |
	byteCodes := self getCoveredByteCodes.
	byteCodes := self computeOchiaiFor: byteCodes.
	self highlightAnomaliesAtStatements: byteCodes.! !

!SDebuggerCodePaneMorph methodsFor: 'accessing' stamp: 'mp 7/16/2013 16:22:09.525'!
faultTracer
	self flag: #mpe. "ToDo: Refactor!! Law of Demeter is broken!!"
	^ self callNode callTreeModel pathFinder coverageTracer! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 9/21/2011 12:10:42.082'!
getCoveredByteCodes
	| coveredTests wrapper |
	coveredTests := self faultTracer getCoveringTests: self callNode methodReference.
	[wrapper := SMethodByteCodeCoverageForAllTestsWrapper 
					on: self callNode methodReference.
	wrapper install.
	self runTests: coveredTests forAnalyzing: wrapper]
		ensure: [wrapper uninstall].
	^ wrapper coveredByteCodes! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 7/16/2013 11:35:32.909'!
highlightAnomaliesAtStatements: suspiciousByteCodes
	| method map stream range |
	method := self callNode compiledMethod.
	map := (DebuggerMethodMap forMethod: method).
	suspiciousByteCodes do: [:each | 
		stream := (InstructionStream on: method).
		stream pc: each key.
		(stream willReallySend or: [stream willReturn])
			ifTrue: [range := map rangeForPC: each key contextIsActiveContext: true.
					self definition text 
						addAttribute: (TextColor 
										color: (Color 
												h: 120 * (1.0 - each value first) 
												s: each value second
												v: 1.0)) 
						from: range first 
						to: range last;
						addAttribute: TextEmphasis underlined
						from: range first 
						to: range last]].
			self definition selectAll; changed.! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 9/21/2011 12:25:14.785'!
ochiaiForByteCode: aByteCode from: aDictionary
	| coveredTests failuresAtByteCode passedAtByteCode allFailures suspiciousness confidence |
	allFailures := self faultTracer allFailures.
	coveredTests := aDictionary keys 
							select: [:test | (aDictionary at: test) includes: aByteCode].
	failuresAtByteCode := self faultTracer 	failuresFrom: coveredTests.		
	passedAtByteCode := coveredTests size - failuresAtByteCode.
	(allFailures = 0 or: [failuresAtByteCode = 0]) 
		ifTrue: [suspiciousness := 0.0. 
			     confidence := 1.0]
		ifFalse: [suspiciousness := (failuresAtByteCode / 
				((allFailures * (failuresAtByteCode + passedAtByteCode)) sqrt)). 
				confidence := (failuresAtByteCode / allFailures).].
	^ aByteCode -> {suspiciousness. confidence}! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 9/21/2011 12:10:33.958'!
runTests: aCollectionOfTests forAnalyzing: aWrapper
	aCollectionOfTests 
		do: [:each | aWrapper currentTest: each.
				     each execute] 
		displayingProgress: [:each | self displayTest: each from: aCollectionOfTests size]! !

!SDebuggerCodePaneMorph methodsFor: 'accessing' stamp: 'mp 11/8/2010 16:14:28.07'!
suspiciousColor
	"Answer the value of suspiciousColor"

	^ suspiciousColor! !

!SDebuggerCodePaneMorph methodsFor: 'accessing' stamp: 'mp 11/8/2010 16:14:28.075'!
suspiciousColor: anObject
	"Set the value of suspiciousColor"

	suspiciousColor := anObject! !

SDefinitionModel subclass: #SCallNodeDefinitionModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-CallTree'!

!SCallNodeDefinitionModel commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SCallNodeDefinitionModel class methodsFor: 'instance creation' stamp: 'mp 7/16/2013 17:37:24.036'!
on: aCallTreeModel

	^ self new
		callTreeModel: aCallTreeModel! !

!SCallNodeDefinitionModel methodsFor: 'updating' stamp: 'mp 1/31/2010 18:31'!
update: aCallNode

	(aCallNode notNil and: [aCallNode isLiteral]) ifTrue: [^ self].
	aCallNode ifNil:[^ self setClass: nil andSelector: nil]. 
	self setClass: aCallNode clientClass andSelector: aCallNode clientSelector.	
! !

ObjectExplorer subclass: #SObjectExplorer
	instanceVariableNames: 'codePane'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-CallTree'!

!SObjectExplorer commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SObjectExplorer class methodsFor: 'instance creation' stamp: 'mp 4/27/2010 19:55'!
on: aCodePane

	^ self new
		codePane: aCodePane;
		yourself! !

!SObjectExplorer methodsFor: 'accessing' stamp: 'mp 4/27/2010 19:55'!
codePane
	"Answer the value of codePane"

	^ codePane! !

!SObjectExplorer methodsFor: 'accessing' stamp: 'mp 4/27/2010 19:55'!
codePane: anObject
	"Set the value of codePane"

	codePane := anObject! !

!SObjectExplorer methodsFor: 'menus' stamp: 'mp 7/7/2010 16:46:23.101'!
genericMenu: aMenu 
	|  menu morph |
	currentSelection
		ifNil: [menu := aMenu.
			menu
				add: '*nothing selected*'
				target: self
				selector: #yourself]
		ifNotNil: [
			morph := SObjectExplorationMorph 
						on: self codePane 
						title: currentSelection asString
						object: currentSelection 
						color: self codePane color.
			self codePane addObjectExplorationMorph: morph].
	^ aMenu! !

!SObjectExplorer methodsFor: 'accessing' stamp: 'mp 4/27/2010 20:15'!
getList
	^ (ObjectExplorerWrapper with: rootObject name: '' model: nil) contents! !

!SObjectExplorer methodsFor: 'as yet unclassified' stamp: 'mp 4/27/2010 18:39'!
noteNewSelection: anObject

	currentSelection := anObject.! !

SActionButtonMorph subclass: #STestResultFeedbackMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-TestChooser'!

!STestResultFeedbackMorph commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!STestResultFeedbackMorph class methodsFor: 'constants' stamp: 'mp 4/15/2010 16:08'!
defaultColor

	^ Color white! !

!STestResultFeedbackMorph class methodsFor: 'constants' stamp: 'mp 4/15/2010 16:08'!
errorColor

	^ Color r: 230 / 256.0 g: 59 / 256.0 b: 59 / 256.0 ! !

!STestResultFeedbackMorph class methodsFor: 'constants' stamp: 'mp 4/15/2010 16:08'!
failureColor

	^ Color r: 245 / 256.0 g: 237 / 256.0 b: 13 / 256.0 ! !

!STestResultFeedbackMorph class methodsFor: 'constants' stamp: 'mp 4/15/2010 16:09'!
passedColor

	^ Color r: 70 / 256.0 g: 189 / 256.0 b: 47 / 256.0 ! !

!STestResultFeedbackMorph methodsFor: 'initialization' stamp: 'mp 4/15/2010 16:15'!
initialize

	super initialize.
	self color: self class defaultColor.! !

!STestResultFeedbackMorph methodsFor: 'updating' stamp: 'mp 5/2/2010 12:52'!
update: aResult

	(aResult isNil or: [aResult isLiteral]) ifTrue: [^ self]. 
	(aResult isKindOf: TestResult) not ifTrue: [^ self].
	
	aResult passed ifNotEmpty: [self color: self class passedColor].
	aResult failures ifNotEmpty: [self color: self class failureColor].
	aResult errors ifNotEmpty: [self color: self class errorColor].! !

BreakpointManager subclass: #SBreakpointManager
	instanceVariableNames: ''
	classVariableNames: 'InstalledNodes'
	poolDictionaries: ''
	category: 'Path-Finder-Debugger'!

!SBreakpointManager commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SBreakpointManager class methodsFor: 'as yet unclassified' stamp: 'mp 3/10/2010 18:48'!
compilePrototype: aSymbol in: aClass 
	"Compile and return a new method containing a break statement"

	| source node trailer method |
	source := self breakpointMethodSourceFor: aSymbol in: aClass.
	node := aClass compilerClass new
		compile: source
		in: aClass 
		notifying: nil 
		ifFail: [self error: '[breakpoint] unable to install breakpoint'].
	node isNil ifTrue: [^nil].
	trailer := CompiledMethodTrailer new.
	trailer sourceCode: source.
	method := node generate: trailer.
	self installedNodes at: method put: node.
	^ method! !

!SBreakpointManager class methodsFor: 'as yet unclassified' stamp: 'mp 3/10/2010 18:29'!
installedNodes
	InstalledNodes isNil ifTrue:[InstalledNodes := IdentityDictionary new].
	^InstalledNodes! !

!SBreakpointManager class methodsFor: 'as yet unclassified' stamp: 'mp 3/10/2010 18:40'!
nukeInstalledNodes

	"self nukeInstalledNodes"
	InstalledNodes := nil.! !

!SBreakpointManager class methodsFor: 'as yet unclassified' stamp: 'mp 3/10/2010 18:41'!
unInstall: breakMethod 

	self installedNodes removeKey: breakMethod.
	super unInstall: breakMethod.! !

Object subclass: #SCallTreeTracer
	instanceVariableNames: 'project selectedCategories callTreeModel profilingMode viewMode active currentId withContracts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SCallTreeTracer commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SCallTreeTracer class methodsFor: 'instance creation' stamp: 'mp 3/27/2012 16:12:29.794'!
on: aSProject

	^ self new
		project: aSProject;
		selectedCategories: aSProject categories! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:33'!
active
	^ active! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:33'!
active: anObject
	active := anObject! !

!SCallTreeTracer methodsFor: 'callTree' stamp: 'mp 7/16/2013 16:35:10.018'!
addProfileInformation: aValue node: anId

	(self getNode: anId) profiles at: self profilingMode put: aValue.! !

!SCallTreeTracer methodsFor: 'callTree' stamp: 'mp 7/16/2013 16:35:16.19'!
addView: aString to: aSymbol

	self callTreeModel currentNode views at: aSymbol put: aString.! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 11/29/2012 15:59:46.024'!
askForContracts
	self project hasContracts ifFalse: [^ false].
	self withContracts ifTrue: [^ true].
	^ UIManager default 
		chooseFrom: {'Yes'. 'No'}
		values: {true. false}
		lines: #() 
		title: 'Run with Contracts?'! !

!SCallTreeTracer methodsFor: 'specific runs' stamp: 'mp 11/8/2010 15:17:54.236'!
byteCodeCoverageAt: aCallNode
	| wrapper |
	wrapper := self runTestWithSpecificWrapper: (
					self createByteCodeCoverageWrapperOn: aCallNode).
	^ wrapper coveredByteCodes
! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 7/16/2013 16:24:04.205'!
callTreeModel

	^ callTreeModel! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 7/16/2013 16:24:04.21'!
callTreeModel: anObject

	callTreeModel := anObject! !

!SCallTreeTracer methodsFor: 'specific runs' stamp: 'mp 7/16/2013 16:25:26.096'!
createByteCodeCoverageWrapperOn: aCallNode

	^ SByteCodeCoverageWrapper 
			on: aCallNode clientSelector 
			inClass: aCallNode clientClass 
			waitingCalls: (self callTreeModel numberOfCallsBefore: aCallNode)! !

!SCallTreeTracer methodsFor: 'callTree' stamp: 'mp 7/16/2013 16:25:33.568'!
createCodeRootNode: aSourceCodeString

	self callTreeModel rootNode: (SRootCodeCallNode 
								id: self rootId
								code: aSourceCodeString
								fromSender: nil
								model: self callTreeModel).
	self currentId: self rootId.
	self callTreeModel currentNode: self callTreeModel rootNode.! !

!SCallTreeTracer methodsFor: 'specific runs' stamp: 'mp 7/16/2013 16:25:38.542'!
createDebugWrapperOn: aCallNode

	^ SDebugWrapper 
			on: aCallNode clientSelector 
			inClass: aCallNode clientClass 
			waitingCalls: (self callTreeModel numberOfCallsBefore: aCallNode)! !

!SCallTreeTracer methodsFor: 'specific runs' stamp: 'mp 7/7/2010 17:21:11.71'!
createExploreWrapper: aSymbol on: aCallNode 
	
	^ SExploreWrapper 
		on: aCallNode clientSelector
		inClass: aCallNode clientClass
		waitingCalls: aCallNode callsBefore
		exploreWhat: aSymbol! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 7/16/2013 16:25:48.28'!
createNewCallNodeOn: aClass with: aSelector receiverType: aTypeClass callsBefore: aNumber

	| node |
	node := SCallNode 
			id: self nextId 
			class: aClass
			methodName: aSelector
			fromSender: self callTreeModel currentNode
			model: self callTreeModel
			receiverType: aTypeClass
			callsBefore: aNumber.
	self callTreeModel currentNode addCall: node.
	self callTreeModel currentNode: node.
	self callTreeModel currentNode 
		selectedView: self viewMode;
		selectedProfile: self profilingMode.
	^ node id! !

!SCallTreeTracer methodsFor: 'callTree' stamp: 'mp 7/16/2013 16:36:45.369'!
createNewCallTreeOn: aTestCaseClass with: aMethodSelector

	self 
		resetCallTree;
		createRootNode: aTestCaseClass withTest: aMethodSelector.! !

!SCallTreeTracer methodsFor: 'callTree' stamp: 'mp 7/16/2013 16:37:47.66'!
createNewCallTreeOnCode: aSourceCodeString

	self 
		resetCallTree;
		createCodeRootNode: aSourceCodeString.! !

!SCallTreeTracer methodsFor: 'callTree' stamp: 'mp 7/16/2013 16:25:55.342'!
createRootNode: aTestClass withTest: aSelector

	self callTreeModel rootNode: (SRootTestCallNode 
								id: self rootId
								class: (aTestClass lookupSelector: aSelector) 
										methodClass
								methodName: aSelector 
								fromSender: nil
								model: self callTreeModel
								receiverType: aTestClass
								callsBefore: 0).
	self currentId: self rootId.
	self callTreeModel currentNode: self callTreeModel rootNode.! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 5/28/2013 10:33:54.16'!
createWrapperOn: aClass and: aSelector addTo: aCollection
	| wrapper |
	(self isOnBlacklist: (MethodReference class: aClass selector: aSelector))
		ifFalse: [wrapper := self getCallWrapperClass 
								on: aSelector inClass: aClass withTracer: self.
				wrapper ifNotNil: [
					wrapper selectedView: self viewMode.
					aCollection add: wrapper.
					wrapper install]].! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 5/28/2013 10:14:09.023'!
createWrappers
	| wrappers |
	wrappers := OrderedCollection new.
	self selectedClasses do: [:klass | 
			klass selectors do: [:selector | 
				self createWrapperOn: klass and: selector addTo: wrappers].		
			klass class selectors do: [:selector | 
				self createWrapperOn: klass class and: selector addTo: wrappers]].
	^ wrappers! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:59'!
currentId
	^ currentId! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:59'!
currentId: anObject
	currentId := anObject! !

!SCallTreeTracer methodsFor: 'specific runs' stamp: 'mp 7/16/2013 16:26:00.833'!
debugAt:	aCallNode
	
	| wrapper |
	(self callTreeModel numberOfCallsBefore: aCallNode) = 0
		ifTrue: [
			[SBreakpointManager installInClass: aCallNode clientClass selector: aCallNode clientSelector.
			self runRootNode] ensure: [
				SBreakpointManager unInstall:  aCallNode clientClass >> aCallNode clientSelector]]
		ifFalse: [
			[wrapper := self createDebugWrapperOn: aCallNode.
			wrapper install.
			self runRootNode] ensure: [
				SBreakpointManager unInstall:  aCallNode clientClass >> aCallNode clientSelector.
				wrapper uninstall]]! !

!SCallTreeTracer methodsFor: 'specific runs' stamp: 'bs 7/17/2010 14:25:54.337'!
explore: aSymbol on: aCallNode

	| instVars |
	instVars := self runTestWithSpecificWrapper: (self createExploreWrapper: aSymbol on: aCallNode).
	^ instVars objects! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 7/7/2010 17:21:52.072'!
getCallWrapperClass

	(#(% ms) includes: self profilingMode)
		ifTrue: [^ SProfilingWrapper].
	(#(identities objects) includes: self viewMode)
		ifTrue: [^ SViewWrapper].
	^ SCallWrapper! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 7/16/2013 16:26:15.92'!
getNode: aNumber

	self callTreeModel currentNode id = aNumber
		ifTrue: [^ self callTreeModel currentNode]
		ifFalse: [^ self callTreeModel 
					findNodeInSubGraph: aNumber 
					startNode: self callTreeModel rootNode].! !

!SCallTreeTracer methodsFor: '-- all --' stamp: 'mp 3/5/2010 17:53'!
initialize

	self 
		stopTracing;
		viewMode: #classes;
		profilingMode: #normal;
		currentId: -1.! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 18:08'!
isActive
	
	^ self active! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 7/16/2013 15:59:10.806'!
isOnBlacklist: aMethodReference
	| blacklist |
	blacklist := {MwMethodWrapper. SCallTreeTracer. SCallTreeModel. SNode}.
	^ blacklist anySatisfy: [:each | 
		(aMethodReference actualClass includesBehavior: each) or: 
			[aMethodReference actualClass includesBehavior: each class]].! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 17:26'!
nextId
	
	self currentId: self currentId + 1.
	^ self currentId! !

!SCallTreeTracer methodsFor: 'profiling' stamp: 'mp 7/16/2013 17:03:50.859'!
postProfilingCounts

	| bag |
	bag := Bag new.
	self callTreeModel 
		traverseCallTree: self callTreeModel rootNode 
		with: [:node | bag add: node clientClass -> node clientSelector.
			node profiles 
				at: #counts 
				put: '#', (bag occurrencesOf: node clientClass -> node clientSelector) asString].! !

!SCallTreeTracer methodsFor: 'profiling' stamp: 'mp 7/16/2013 17:03:52.612'!
postProfilingMilliSeconds

	| duration |
	self callTreeModel 
		traverseCallTree: self callTreeModel rootNode 
		with: [:node | 
			duration := node profiles at: #ms.
			node profiles at: #ms put: duration asMilliSeconds asString, 'ms'].! !

!SCallTreeTracer methodsFor: 'profiling' stamp: 'mp 7/16/2013 17:03:54.921'!
postProfilingPercentage

	| fullDuration duration |
	fullDuration := self callTreeModel startingCallNodes 
					inject: 0 into: [:sum :each | sum := sum + (each profiles at: #%) asNanoSeconds].
					
	self callTreeModel 
		traverseCallTree: self callTreeModel rootNode 
		with: [:node | 
			(node profiles includesKey: #%) 
				ifTrue: [duration := (node profiles at: #%) asNanoSeconds.
					node profiles at: #% put: ((duration / fullDuration) * 100.0) rounded asString, '%']].! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 17:11'!
profilingMode
	^ profilingMode ifNil: [profilingMode := #normal]! !

!SCallTreeTracer methodsFor: 'public API' stamp: 'mp 1/18/2010 16:31'!
profilingMode: anObject
	profilingMode := anObject! !

!SCallTreeTracer methodsFor: 'profiling' stamp: 'mp 1/25/2010 15:47'!
profilingPostProcessing

	self profilingMode == #ms
		ifTrue: [self postProfilingMilliSeconds].
	self profilingMode == #%
		ifTrue: [self postProfilingPercentage].
	self profilingMode == #counts
		ifTrue: [self postProfilingCounts].! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 3/27/2012 16:11:51.655'!
project
	"Answer the value of project"

	^ project! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 3/27/2012 16:11:51.668'!
project: anObject
	"Set the value of project"

	project := anObject! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 1/18/2010 17:17'!
removeWrappers: aWrapperCollection
	
	aWrapperCollection do: [:each | each uninstall].! !

!SCallTreeTracer methodsFor: 'public API' stamp: 'mp 7/16/2013 16:35:38.034'!
resetCallTree

	self currentId: -1.
	self callTreeModel 
		reset; 
		refresh.! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 7/16/2013 16:27:26.978'!
returnFromNode: id

	| node |
	self callTreeModel currentNode id = id
		ifTrue: [self callTreeModel 
					currentNode: self callTreeModel currentNode fromSender]
		ifFalse: [node := self callTreeModel 
								findNodeInSubGraph: id 
								startNode: self callTreeModel rootNode.
				self callTreeModel currentNode: node fromSender].! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:46'!
rootId

	^ 0! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 7/16/2013 16:27:33.793'!
runRootNode
	^ self callTreeModel rootNode run! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 7/16/2013 16:27:36.112'!
runRootNodeWithContracts
	^ [[self runRootNode] runWithContracts: self project projectName]
		on: SContractViolation
		do: [:exception | 
			self callTreeModel currentNode violations add: exception.
			exception resume]! !

!SCallTreeTracer methodsFor: 'specific runs' stamp: 'mp 7/12/2010 11:37:26.051'!
runTestWithSpecificWrapper: aWrapper

	aWrapper install.
	[self runRootNode]
		ensure: [aWrapper uninstall.
			      ^ aWrapper].! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:29'!
selectedCategories
	^ selectedCategories! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:29'!
selectedCategories: anObject
	selectedCategories := anObject! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 1/18/2010 16:30'!
selectedClasses

	^ Smalltalk allClasses select: [:aClass | self selectedCategories includes: aClass category]! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 1/18/2010 17:32'!
startTracing

	self active: true.! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 1/18/2010 17:32'!
stopTracing

	self active: false.! !

!SCallTreeTracer methodsFor: 'tracing' stamp: 'mp 7/16/2013 16:27:41.755'!
trace
	| result wrappers runWithContracts |
	wrappers := self createWrappers.
	runWithContracts := self askForContracts.
	self startTracing.
	result := runWithContracts
		ifTrue: [self runRootNodeWithContracts]
		ifFalse: [self runRootNode].
	self 
		stopTracing;
		removeWrappers: wrappers;
		profilingPostProcessing.
	self callTreeModel refresh.
	^ result! !

!SCallTreeTracer methodsFor: 'public API' stamp: 'mp 7/16/2013 16:37:34.376'!
traceSourceCode: aSourceCodeString

	self createNewCallTreeOnCode: aSourceCodeString.
	^ self trace! !

!SCallTreeTracer methodsFor: 'public API' stamp: 'mp 7/16/2013 16:37:01.096'!
traceTest: aTestCaseClass method: aMethodSelector

	self createNewCallTreeOn: aTestCaseClass with: aMethodSelector.
	^ self trace! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 17:18'!
viewMode
	^ viewMode ifNil: [viewMode := #classes]! !

!SCallTreeTracer methodsFor: 'public API' stamp: 'mp 1/18/2010 16:31'!
viewMode: anObject
	viewMode := anObject! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 11/29/2012 16:00:08.335'!
withContracts
	"Answer the value of withContracts"

	^ withContracts ifNil: [withContracts := false]! !

!SCallTreeTracer methodsFor: 'accessing' stamp: 'mp 11/29/2012 15:56:11.979'!
withContracts: anObject
	"Set the value of withContracts"

	withContracts := anObject! !

Object subclass: #SNode
	instanceVariableNames: 'callTreeModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-CallTree'!

!SNode commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

SNode subclass: #SMethodNode
	instanceVariableNames: 'clientClass clientSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-CallTree'!

!SMethodNode commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

SMethodNode subclass: #SCallNode
	instanceVariableNames: 'id fromSender nextCalls selectedView selectedProfile views profiles objects callsBefore violations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-CallTree'!

!SCallNode commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SCallNode class methodsFor: 'instance creation' stamp: 'mp 7/16/2013 17:22:57.998'!
id: aNumber class: aClass methodName: aSelector fromSender: aCallNode model: aCallTreeModel receiverType: aTypeClass callsBefore: aCallsNumber

	^ (self forClass: aClass withSelector: aSelector model: aCallTreeModel)
		id: aNumber;
		fromSender: aCallNode;
		callsBefore: aCallsNumber;
		setReceiverType: aTypeClass;
		initClassView;
		yourself! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
addCall: aCallNode

	self nextCalls add: aCallNode.! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 6/15/2010 15:59'!
callsBefore
	"Answer the value of callsBefore"

	^ callsBefore! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 6/15/2010 15:59'!
callsBefore: anObject
	"Set the value of callsBefore"

	callsBefore := anObject! !

!SCallNode methodsFor: 'callgraph' stamp: 'mp 4/24/2010 18:43'!
clearObjectCache

	self objects: nil.! !

!SCallNode methodsFor: 'callgraph' stamp: 'mp 4/24/2010 18:51'!
explore: aSymbol
	^ self objects 
		at: aSymbol 
		ifAbsentPut: [self tracer explore: aSymbol on: self]! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
fromSender
	^ fromSender! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
fromSender: anObject
	fromSender := anObject! !

!SCallNode methodsFor: 'callgraph' stamp: 'mp 1/25/2010 13:59'!
getCallStack

	| callStack currentNode |
	callStack := OrderedCollection new.
	currentNode := self.
	callStack addFirst: currentNode.
	[currentNode fromSender notNil] whileTrue: [
		currentNode := currentNode fromSender.
		callStack addFirst: currentNode].
	^ callStack! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 3/27/2012 17:14:05.732'!
hasViolations
	"Direct access to prevent lazy initialize"

	^ violations notNil and: [violations notEmpty]! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
id
	^ id! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
id: anObject
	id := anObject! !

!SCallNode methodsFor: 'initialize-release' stamp: 'mp 5/5/2010 13:41'!
initClassView

	self views at: #classes put: self clientClass.! !

!SCallNode methodsFor: 'initialize-release' stamp: 'mp 5/5/2010 13:37'!
initialize

	self id: -1.
	self nextCalls: OrderedCollection new.
	self views: Dictionary new.
	self profiles: Dictionary new.
	self selectedView: #classes.
	self selectedProfile: #normal.! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
nextCalls
	^ nextCalls! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
nextCalls: anObject
	nextCalls := anObject! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 4/19/2010 20:38'!
objects
	"Answer the value of objects"

	^ objects ifNil: [objects := Dictionary new]! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 4/19/2010 20:37'!
objects: anObject
	"Set the value of objects"

	objects := anObject! !

!SCallNode methodsFor: 'printing' stamp: 'mp 4/24/2010 14:04'!
printNodeId

	^ '(', self id asString, ')'	! !

!SCallNode methodsFor: 'printing' stamp: 'mp 1/25/2010 13:57'!
printProfile

	self selectedProfile == #normal ifTrue: [^''].
	^ '[',  (self profiles at: self selectedProfile), ']'! !

!SCallNode methodsFor: 'printing' stamp: 'mp 4/24/2010 14:04'!
printSelector

	^ '>>', self clientSelector! !

!SCallNode methodsFor: 'printing' stamp: 'mp 7/16/2013 16:20:16.136'!
printString

	 ^ (self callTreeModel printInfosForNode: self), 
		self printProfile, 
		self printView, 
		self printSelector

	! !

!SCallNode methodsFor: 'printing' stamp: 'mp 2/1/2012 11:21:55.809'!
printView

	| result |
	(self selectedView == #classes 
		and: [((self views at: #classes) asString = (self views at: #receiver) asString) not]) 
			ifTrue: [^ (self views at: #receiver) asString, 
						'(', (self views at: #classes) asString, ')'].
	self selectedView == #identities
		ifTrue: [result := (self views at: #receiver) asString, 
					'(', (self views at: #identities) asString, ')'.
				((self views at: #classes) asString = 
					(self views at: #receiver) asString) not
						ifTrue: [result := result, '(', (self views at: #classes) asString, ')'].
				^ result	].
	^ (self views at: self selectedView) asString.

	! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
profiles
	"Answer the value of profiles"

	^ profiles! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
profiles: anObject
	"Set the value of profiles"

	profiles := anObject! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 5/14/2012 10:56:47.08'!
receiverClass
	^ self views at: #receiver! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
selectedProfile
	"Answer the value of selectedProfile"

	^ selectedProfile! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
selectedProfile: anObject
	"Set the value of selectedProfile"

	selectedProfile := anObject! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
selectedView
	"Answer the value of selectedView"

	^ selectedView! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
selectedView: anObject
	"Set the value of selectedView"

	selectedView := anObject! !

!SCallNode methodsFor: 'callgraph' stamp: 'mp 7/16/2013 16:20:19.335'!
setAsCurrentSelection

	self callTreeModel 
		newSelectedItem: self;
		changed: #getSelectedPath.! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 5/5/2010 13:25'!
setReceiverType: aClass

		self views at: #receiver put: aClass.! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 7/16/2013 16:20:22.405'!
tracer

	^ self callTreeModel tracer! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
views
	"Answer the value of views"

	^ views! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
views: anObject
	"Set the value of views"

	views := anObject! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 3/27/2012 16:56:00.054'!
violations
	"Answer the value of violations"

	^ violations ifNil: [violations := OrderedCollection new]! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 3/27/2012 16:55:17.287'!
violations: anObject
	"Set the value of violations"

	violations := anObject! !

SCallNode subclass: #SRootCallNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-CallTree'!

!SRootCallNode commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SRootCallNode methodsFor: 'printing' stamp: 'mp 4/27/2010 20:28'!
printString

	 ^ 'Entry Point'! !

!SRootCallNode methodsFor: 'running' stamp: 'mp 5/2/2010 12:28'!
run

	self subclassResponsibility.! !

SRootCallNode subclass: #SRootCodeCallNode
	instanceVariableNames: 'code'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-CallTree'!

!SRootCodeCallNode commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SRootCodeCallNode class methodsFor: 'instance creation' stamp: 'mp 7/16/2013 17:23:17.209'!
id: aNumber code: aSourceCodeString fromSender: aCallNode model: aCallTreeModel

	^ (self forClass: nil withSelector: nil model: aCallTreeModel)
		id: aNumber;
		fromSender: aCallNode;
		code: aSourceCodeString;
		initClassView;
		yourself! !

!SRootCodeCallNode methodsFor: 'accessing' stamp: 'mp 5/2/2010 12:30'!
code
	"Answer the value of code"

	^ code! !

!SRootCodeCallNode methodsFor: 'accessing' stamp: 'mp 5/2/2010 12:30'!
code: anObject
	"Set the value of code"

	code := anObject! !

!SRootCodeCallNode methodsFor: 'running' stamp: 'mp 5/2/2010 12:55'!
numArgsOfMethod
	"Custom code has no method and no arguments"
	^ 0! !

!SRootCodeCallNode methodsFor: 'running' stamp: 'mp 5/2/2010 12:51'!
run
	^ Compiler evaluate: self code! !

SRootCallNode subclass: #SRootTestCallNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-CallTree'!

!SRootTestCallNode commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SRootTestCallNode methodsFor: 'printing' stamp: 'mp 7/12/2013 10:15:01.159'!
printString
	| signal result |
	signal := SwaTestExceptionRegistry 
		at: (self clientClass selector: self clientSelector) 
		ifAbsent: nil.
	result := 'Entry Point'.
	signal ifNotNil: [result := result, ' - ', signal asString].
	 ^ result! !

!SRootTestCallNode methodsFor: 'running' stamp: 'mp 7/11/2013 10:23:13.89'!
run
	| result |
	 self receiverClass resources do: [:each |
		each isAvailable ifFalse: [ each signalInitializationError ]].
	[result := (self receiverClass selector: self clientSelector) runWithoutTimeout] 
		ensure: [self receiverClass resources do: [:each | each reset ]].
	^ result! !

!SMethodNode class methodsFor: 'instance creation' stamp: 'mp 7/16/2013 17:23:09.86'!
forClass: aClass withSelector: aSelector model: aCallTreeModel

	^ (self for: aCallTreeModel)
			clientClass: aClass;
			clientSelector: aSelector! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 1/3/2010 14:04'!
clientClass
	^ clientClass! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 1/3/2010 14:04'!
clientClass: anObject
	clientClass := anObject! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 1/3/2010 14:04'!
clientSelector
	^ clientSelector! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 1/3/2010 14:04'!
clientSelector: anObject
	clientSelector := anObject! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 4/24/2010 15:04'!
compiledMethod

	^ self clientClass >> self clientSelector! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 4/24/2010 15:03'!
methodReference

	^ MethodReference class: self clientClass selector: self clientSelector! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 5/2/2010 12:54'!
numArgsOfMethod

	^ self compiledMethod numArgs! !

!SNode class methodsFor: 'instance creation' stamp: 'mp 7/16/2013 16:19:23.22'!
for: aCallTreeModel

	^ self new
		callTreeModel: aCallTreeModel! !

!SNode methodsFor: 'accessing' stamp: 'mp 7/16/2013 16:18:56.208'!
callTreeModel

	^ callTreeModel! !

!SNode methodsFor: 'accessing' stamp: 'mp 7/16/2013 16:18:56.214'!
callTreeModel: anObject

	callTreeModel := anObject! !

Object subclass: #STestChooserModel
	instanceVariableNames: 'tracer test result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-TestChooser'!

!STestChooserModel commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!STestChooserModel class methodsFor: 'instance creation' stamp: 'mp 4/15/2010 15:32'!
on: aTracer

	^ self new
		tracer: aTracer;
		yourself! !

!STestChooserModel methodsFor: 'private' stamp: 'mp 5/2/2010 12:25'!
analyzeTest

	| result |
	result := self test isString
		ifTrue: [self tracer traceSourceCode: self test]
		ifFalse: [self tracer traceTest: self test actualClass method: self test methodSymbol].
	self result: result.! !

!STestChooserModel methodsFor: 'spec commands' stamp: 'mp 4/15/2010 15:39'!
isTestAvailable

	^ self test notNil! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:57'!
normalRun

	self setProfilingStateTo: #normal.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:57'!
normalRunWithCounts	
	
	self setProfilingStateTo: #counts.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:57'!
profileRunWithMs

	self setProfilingStateTo: #ms.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:57'!
profileRunWithPercentage

	self setProfilingStateTo: #%.! !

!STestChooserModel methodsFor: 'private' stamp: 'mp 7/16/2013 17:24:18.553'!
resetCallTree

	self tracer ifNil: [^ self].
	self tracer resetCallTree.! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 1/21/2010 12:39'!
result
	"Answer the value of result"

	^ result! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 4/15/2010 16:21'!
result: anObject
	"Set the value of result"

	result := anObject.
	self changed: result.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:58'!
selectClassesView

	self setViewStateTo: #classes.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:58'!
selectIdentityView

	self setViewStateTo: #identities.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:58'!
selectObjectView

	self setViewStateTo: #objects.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 7/16/2013 17:24:31.243'!
setCurrentCode: aSourceCodeString

	self test: aSourceCodeString.
	self changed: #isTestAvailable.
	aSourceCodeString 
		ifNil: [self resetCallTree.
	 		   ^ self].
	self analyzeTest.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 7/16/2013 17:24:33.616'!
setCurrentTest: aMethodReference

	self test: aMethodReference.
	self changed: #isTestAvailable.
	aMethodReference 
		ifNil: [self resetCallTree.
	 		   ^ self].
	self analyzeTest.	! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:56'!
setProfilingStateTo: aSymbol 
	
	self tracer profilingMode: aSymbol.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:57'!
setViewStateTo: aSymbol 
	
	self tracer viewMode: aSymbol.! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 4/15/2010 15:38'!
test
	"Answer the value of test"

	^ test! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 4/15/2010 15:38'!
test: anObject
	"Set the value of test"

	test := anObject! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 1/21/2010 09:59'!
tracer
	"Answer the value of tracer"

	^ tracer! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 1/21/2010 09:59'!
tracer: anObject
	"Set the value of tracer"

	tracer := anObject! !

STreeModel subclass: #SCallTreeModel
	instanceVariableNames: 'rootNode currentNode history currentHistoryNode tracer searchQuery pathFinder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-CallTree'!

!SCallTreeModel commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SCallTreeModel class methodsFor: 'instance creation' stamp: 'mp 3/25/2011 12:46:16.783'!
on: aTracer

	^ self on: aTracer pathFinder: nil! !

!SCallTreeModel class methodsFor: 'instance creation' stamp: 'mp 3/25/2011 12:45:54.853'!
on: aTracer pathFinder: aPathFinder

	^ self new
		tracer: aTracer;
		pathFinder: aPathFinder.! !

!SCallTreeModel methodsFor: 'history' stamp: 'mp 1/25/2010 16:19'!
addToHistory: aNode

	aNode ifNotNil: [
		(self history notEmpty and: [((self history at: self currentHistoryNode) id = aNode id)])
			ifTrue: [^ self].			
		(self history size = self currentHistoryNode) not
			ifTrue: [self history: (self history copyUpTo: (self history at: self currentHistoryNode + 1))].
		self history add: aNode.
		self currentHistoryNode: self history size].
	self refreshHistoryButtons.! !

!SCallTreeModel methodsFor: 'menu' stamp: 'mp 8/8/2013 15:54:35.481'!
browseMethod
	| browser colors |
	self hasSelectedItem 
		ifTrue:[browser := SBrowser fullOnClass: self selectedItem clientClass 
					       selector: self selectedItem clientSelector.
				colors := self collectAllColorsFromNodes.
				colors
					ifNotEmpty: [browser methodColors: colors].
				^ browser]! !

!SCallTreeModel methodsFor: 'menu' stamp: 'mp 7/16/2013 17:00:21.503'!
callTreeMenu: aMenu

	^ aMenu
		title: 'Node and Tree Actions';
		add: 'Browse' action: #browseMethod;
		add: 'Debug' action: #debugMethod;
		addLine;
		add: 'Expand Tree' action: #expandSelectedNode;
		yourself.! !

!SCallTreeModel methodsFor: 'searching' stamp: 'mp 1/16/2010 18:42'!
clearHighlighting

	self highlightedItems: nil! !

!SCallTreeModel methodsFor: 'tree functions' stamp: 'mp 8/8/2013 14:47:58.886'!
collectAllColorsFromNodes
	| methodColors |
	methodColors := self getAllNodes inject: Dictionary new into: [:dict :each | 
		dict 
			at: each methodReference put: (self getColorFromMap: each); 
			yourself].
	^ methodColors select: [:each | each value notNil and: [each value ~= Color white]]! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 1/15/2010 14:52'!
currentHistoryNode
	^ currentHistoryNode! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 1/15/2010 14:52'!
currentHistoryNode: anObject
	currentHistoryNode := anObject! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 1/3/2010 13:45'!
currentNode
	^ currentNode! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 1/3/2010 13:45'!
currentNode: anObject
	currentNode := anObject! !

!SCallTreeModel methodsFor: 'menu' stamp: 'mp 1/25/2010 16:44'!
debugMethod

	self hasSelectedItem ifTrue: [self tracer debugAt: self selectedItem].! !

!SCallTreeModel methodsFor: 'spec commands' stamp: 'mp 8/13/2013 11:50:58.545'!
drawExclamationMarkOn: icon
	| canvas |
	canvas := FormCanvas on: icon.
	canvas translucentImage: UiFugueIcons exclamationFrameIcon at: 0@0.
	^ canvas form! !

!SCallTreeModel methodsFor: 'tree functions' stamp: 'mp 1/29/2010 10:32'!
expandNode: aNode

	aNode setAsCurrentSelection.
	self changed: #expandAllOfSelectedNode.! !

!SCallTreeModel methodsFor: 'tree functions' stamp: 'mp 1/29/2010 10:15'!
expandRootNode

	self expandNode: self rootNode.! !

!SCallTreeModel methodsFor: 'tree functions' stamp: 'mp 3/4/2010 14:46'!
expandSelectedNode

	self selectedItem
		ifNil: [self expandNode: self rootNode]
		ifNotNil: [self expandNode: self selectedItem]
	! !

!SCallTreeModel methodsFor: 'tree functions' stamp: 'mp 1/3/2010 14:59'!
findNodeInSubGraph: aNumber startNode: aNode

	| lastNode |
	aNode nextCalls do: [:each | 
		each id = aNumber ifTrue: [^ each].
		each id < aNumber ifTrue: [lastNode := each].
		each id > aNumber ifTrue: [
			lastNode	ifNotNil: [^ self findNodeInSubGraph: aNumber startNode: lastNode]]].
	lastNode ifNotNil: [^ self findNodeInSubGraph: aNumber startNode: lastNode].
	^ nil! !

!SCallTreeModel methodsFor: 'spec commands' stamp: 'mp 3/25/2011 16:07:06.532'!
getAdditionalInformationFromMap: aCallNode
	^ self pathFinder coverageTracer 
			ifNotNil: [:tracer | 
				tracer getAdditionalInfoFromMapFor: (MethodReference 
												class: aCallNode clientClass 
												selector: aCallNode clientSelector)].! !

!SCallTreeModel methodsFor: 'tree functions' stamp: 'mp 7/16/2013 17:03:34.162'!
getAllNodes
	| nodes |
	nodes := OrderedCollection new.
	self traverseCallTree: self rootNode with: [:each | nodes add: each].
	^ nodes
! !

!SCallTreeModel methodsFor: 'spec commands' stamp: 'mp 1/3/2010 15:09'!
getChildren: aNode

	^ aNode nextCalls! !

!SCallTreeModel methodsFor: 'path view' stamp: 'mp 7/16/2013 17:03:36.581'!
getClassesFromTrace
	| classes |
	classes := Set new. 
	self traverseCallTree: self rootNode with: [:each | 
		each clientClass isMeta
			ifTrue: [classes add: each clientClass theNonMetaClass]
			ifFalse: [classes add: each clientClass]].
	^ classes! !

!SCallTreeModel methodsFor: 'spec commands' stamp: 'mp 8/8/2013 15:53:08.333'!
getColorFromMap: aCallNode
	self pathFinder ifNil: [^ nil].
	^ self pathFinder coverageTracer 
			ifNotNil: [:tracer | 
				tracer getColorFromMapFor: (MethodReference 
												class: aCallNode clientClass 
												selector: aCallNode clientSelector)].! !

!SCallTreeModel methodsFor: 'spec commands' stamp: 'mp 8/13/2013 11:50:09.057'!
getIcon: aCallNode
	| icon color |	
	color := self getColorFromMap: aCallNode.
	((color isNil or: [color = Color white]) and: [aCallNode hasViolations not]) ifTrue: [^ nil].
	color ifNil: [color := Color white].
	icon := Form extent: 16@15 depth: 32.
	icon fillColor: color.
	aCallNode hasViolations
		ifTrue: [icon := self drawExclamationMarkOn: icon].
	^ icon! !

!SCallTreeModel methodsFor: 'path view' stamp: 'mp 7/16/2013 17:03:38.957'!
getMethodsFromTrace
	| methods |
	methods := Set new.
	self traverseCallTree: self rootNode 
		 with: [:each | methods add: each methodReference].
	^ methods! !

!SCallTreeModel methodsFor: 'searching' stamp: 'mp 3/4/2010 13:57'!
getSearchQuery

	self searchQuery ifNil: [^ ''].
	^ self searchQuery! !

!SCallTreeModel methodsFor: 'spec commands' stamp: 'mp 3/1/2010 17:39'!
getSelectedPath

	^ self selectedItem getCallStack! !

!SCallTreeModel methodsFor: 'spec commands' stamp: 'mp 1/3/2010 15:10'!
hasChildren: aNode

	aNode ifNil: [^ false].
	^ aNode nextCalls notEmpty! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 1/15/2010 13:25'!
history
	^ history! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 1/15/2010 13:25'!
history: anObject
	history := anObject! !

!SCallTreeModel methodsFor: 'history' stamp: 'mp 1/15/2010 16:01'!
historyBack

	self currentHistoryNode: self currentHistoryNode - 1.
	(self history at: self currentHistoryNode) setAsCurrentSelection.
	self refreshHistoryButtons.! !

!SCallTreeModel methodsFor: 'history' stamp: 'mp 1/15/2010 15:12'!
historyBackButtonEnabled

	^ (self currentHistoryNode <= 1) not! !

!SCallTreeModel methodsFor: 'history' stamp: 'mp 1/15/2010 16:01'!
historyPrevious

	self currentHistoryNode: self currentHistoryNode + 1.
	(self history at: self currentHistoryNode) setAsCurrentSelection.
	self refreshHistoryButtons.! !

!SCallTreeModel methodsFor: 'history' stamp: 'mp 1/15/2010 15:10'!
historyPreviousButtonEnabled

	^ self history notEmpty and: [(self currentHistoryNode = self history size) not]! !

!SCallTreeModel methodsFor: 'initialize-release' stamp: 'mp 1/25/2010 13:29'!
initialize
	
	self reset.! !

!SCallTreeModel methodsFor: 'spec commands' stamp: 'mp 7/16/2013 17:00:58.899'!
isTreeAvailable

	^ self rootNode notNil! !

!SCallTreeModel methodsFor: 'spec commands' stamp: 'mp 1/15/2010 13:28'!
newSelectedItem: aNode

	self
		selectedItem: aNode;
		addToHistory: aNode;
		changed: aNode.! !

!SCallTreeModel methodsFor: 'tree functions' stamp: 'mp 7/16/2013 17:03:00.948'!
numberOfCallsBefore: aCallNode

	| numberOfCallsBefore |
	numberOfCallsBefore := 0.
	self traverseCallTree: self rootNode 
		with: [:aNode | 
			((aNode clientClass == aCallNode clientClass) and: 
			[aNode clientSelector == aCallNode clientSelector])
				ifTrue: [
					aNode id < aCallNode id 
						ifTrue: [numberOfCallsBefore := numberOfCallsBefore + 1]]]
		abort: [:aNode | aNode id > aCallNode id].
	^ numberOfCallsBefore! !

!SCallTreeModel methodsFor: 'spec commands' stamp: 'mp 3/7/2012 17:23:05.619'!
openPathViewOnTrace
	self openPathViewWith: self getClassesFromTrace.! !

!SCallTreeModel methodsFor: 'path view' stamp: 'ag 5/11/2012 12:37:25.837'!
openPathViewWith: aSetOfClasses
	
	| morph |
	morph := SPathView open classDiagramMorph.
	self preparePathView: morph with: aSetOfClasses.
	morph
		expandAndToggleLabels;
		layoutGraphForked.
	^ morph! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 3/25/2011 12:45:31.191'!
pathFinder
	"Answer the value of pathFinder"

	^ pathFinder! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 3/25/2011 12:45:31.196'!
pathFinder: anObject
	"Set the value of pathFinder"

	pathFinder := anObject! !

!SCallTreeModel methodsFor: 'path view' stamp: 'mp 7/16/2013 11:33:57.332'!
preparePathView: pathViewMorph with: aSetOfClasses
	"This part is extracted from openPathViewWith: to let subclasses make additional
	preparations, BEFORE the diagram layouts."
	
	pathViewMorph addClasses: aSetOfClasses.
	self getMethodsFromTrace
		collect: [ :each | each actualClass -> each methodSymbol ]
		thenDo: [ :each | pathViewMorph pinMethod: each ].! !

!SCallTreeModel methodsFor: 'printing' stamp: 'mp 10/31/2010 12:19:32.481'!
printInfosForNode: aCallNode

	"template method for printing additional information from the entire graph"
	^ ''! !

!SCallTreeModel methodsFor: 'tree functions' stamp: 'mp 1/3/2010 14:21'!
refresh

	self changed: #rootCallNodes.! !

!SCallTreeModel methodsFor: 'history' stamp: 'mp 1/15/2010 16:00'!
refreshHistoryButtons

	self
		changed: #historyBackButtonEnabled;
		changed: #historyPreviousButtonEnabled.! !

!SCallTreeModel methodsFor: 'initialize-release' stamp: 'mp 3/4/2010 13:10'!
reset
	
	self 
		rootNode: nil;
		currentNode: nil;
		history: OrderedCollection new;
		currentHistoryNode: 0;
		newSelectedItem: nil;
		highlightedItems: nil.! !

!SCallTreeModel methodsFor: 'spec commands' stamp: 'mp 3/1/2010 17:41'!
rootCallNodes

	self rootNode ifNil: [^ ''].
	^ {self rootNode}! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 1/3/2010 13:45'!
rootNode
	^ rootNode! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 1/3/2010 13:45'!
rootNode: anObject
	rootNode := anObject! !

!SCallTreeModel methodsFor: 'searching' stamp: 'mp 7/16/2013 17:03:43.541'!
searchNode: aText

	| foundedNodes |
	self clearHighlighting.
	self rootNode ifNotNil: [
		self searchQuery: aText asString.
		foundedNodes := OrderedCollection new.
		self traverseCallTree: self rootNode 
			with: [:aNode | (aNode asString includesSubString: self searchQuery) 
							ifTrue: [foundedNodes add: aNode]].
		self 
			highlightedItems: foundedNodes;
			changed: #highlightItem:;
			selectNextSearchResult: foundedNodes.
		^ foundedNodes]
	! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 3/4/2010 13:56'!
searchQuery
	"Answer the value of searchQuery"

	^ searchQuery! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 3/4/2010 13:56'!
searchQuery: anObject
	"Set the value of searchQuery"

	searchQuery := anObject! !

!SCallTreeModel methodsFor: 'searching' stamp: 'mp 3/4/2010 13:27'!
selectNextSearchResult: aCollection
	| nextSearchNode |
	aCollection ifEmpty: [^ self].
	self selectedItem 
		ifNil: [nextSearchNode := aCollection first]
		ifNotNil: [nextSearchNode := aCollection 
									detect: [:each | each id > self selectedItem id] 
									ifNone: [aCollection first]].
	nextSearchNode setAsCurrentSelection.! !

!SCallTreeModel methodsFor: 'tree functions' stamp: 'mp 1/4/2010 15:51'!
startingCallNodes

	^ self rootNode nextCalls! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 1/18/2010 17:23'!
tracer
	^ tracer! !

!SCallTreeModel methodsFor: 'accessing' stamp: 'mp 1/18/2010 17:23'!
tracer: anObject
	tracer := anObject! !

!SCallTreeModel methodsFor: 'tree functions' stamp: 'mp 7/16/2013 17:02:22.168'!
traverseCallTree: aNode with: aBlock
	"Prefix Order"
	aNode nextCalls do: [:each | 
		aBlock value: each.
		self traverseCallTree: each with: aBlock].! !

!SCallTreeModel methodsFor: 'tree functions' stamp: 'mp 7/16/2013 17:02:35.471'!
traverseCallTree: aNode with: aBlock abort: anotherBlock
	"Prefix Order"
	aNode nextCalls do: [:each | 
		aBlock value: each.
		(anotherBlock value: each) 
			ifFalse: [self traverseCallTree: each with: aBlock abort: anotherBlock]].! !

SCallTreeModel subclass: #SDebuggerCallTreeModel
	instanceVariableNames: 'suspiciousMethods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Debugger'!

!SDebuggerCallTreeModel commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SDebuggerCallTreeModel class methodsFor: 'constants' stamp: 'mp 7/16/2013 17:23:51.331'!
openedMethodThreshold
	"A method must have at least this suspiciousness-value to be opened in a separate code-box in path view."
	
	^ 0.5! !

!SDebuggerCallTreeModel class methodsFor: 'constants' stamp: 'ag 5/11/2012 11:45:05.946'!
openedMethodsPercentage
	"This is the percentage of methods opened in a path view diagram.
	The most suspicious methods will be opened. The openedMethodThreshold is also taken into account."
	
	^ 0.2! !

!SDebuggerCallTreeModel methodsFor: 'icons' stamp: 'ag 5/2/2012 11:26:45.727'!
getColor: suspiciousMethod
	"suspiciousMethod is not a CallNode, but the result of getSuspiciousnessAndConfidenceFor:"
	
	^ Color 
		h: (1.0 - suspiciousMethod value first) * 120 
		s: suspiciousMethod value second 
		v: 1.0! !

!SDebuggerCallTreeModel methodsFor: 'icons' stamp: 'mp 8/13/2013 11:50:05.477'!
getIcon: aCallNode
	| icon suspiciousMethod |
	suspiciousMethod := self getSuspiciousnessAndConfidenceFor: aCallNode.
	suspiciousMethod ifNil: [^ nil].
	icon := Form extent: 16@15 depth: 32.
	icon fillColor: (self getColor: suspiciousMethod).
	aCallNode hasViolations
		ifTrue: [icon := self drawExclamationMarkOn: icon].					
	^ icon! !

!SDebuggerCallTreeModel methodsFor: 'accessing' stamp: 'mp 3/25/2011 12:25:43.124'!
getSuspiciousnessAndConfidenceFor: aCallNode
	| methodRef |
	methodRef := MethodReference 
					class: aCallNode clientClass selector: aCallNode clientSelector.
	^ self suspiciousMethods 
		detect: [:each | each key = methodRef] 
		ifNone: [nil].! !

!SDebuggerCallTreeModel methodsFor: 'path view' stamp: 'mp 7/16/2013 17:04:09.433'!
getSuspiciousnessMapping
	"Return dictionary mapping from CallNode instances to 2-element-arrays with the suspiciousness-color and the regular suspiciousness-value."
	
	| dict addedCompiledMethods |
	addedCompiledMethods := Set new.
	dict := Dictionary new.
	self
		traverseCallTree: self rootNode
		with: [ :node |
			(addedCompiledMethods includes: node compiledMethod)
				ifFalse: [
					addedCompiledMethods add: node compiledMethod.
					(self getSuspiciousnessAndConfidenceFor: node)
						ifNotNilDo: [ :suspiciousMethod |
							dict at: node put: { self getColor: suspiciousMethod. suspiciousMethod value } ] ] ].
	^ dict! !

!SDebuggerCallTreeModel methodsFor: 'path view' stamp: 'mp 7/16/2013 17:23:43.943'!
preparePathView: morph with: aSetOfClasses
	"Did not see a nice way to divide this method into several shorter methods..."
	
	| mapping callNodes openedMethods |
	super preparePathView: morph with: aSetOfClasses.
	mapping := self getSuspiciousnessMapping.
	"Sort according to suspiciousness values."
	callNodes := mapping keys asSortedCollection: [ :node1 :node2 | (mapping at: node1) second first > (mapping at: node2) second first ].
	openedMethods := (callNodes size * self class openedMethodsPercentage) ceiling. "At least one."
	callNodes withIndexDo: [ :node :index | | suspiciousnessColor method |
		method := node methodReference actualClass -> node methodReference methodSymbol.
		suspiciousnessColor := (mapping at: node) first.
		(index <= openedMethods and: [ (mapping at: node) second first >= self class openedMethodThreshold ]) ifTrue: [
			(morph addMethod: method) color: suspiciousnessColor ]. "Open the code-box."
		morph
			morphForMethod: method
			do: [ :methodMorph | | iconMorph |
				"Do not highlight the whole row, because the view looks to crowded and colorful."
				"methodMorph highlightedColor: suspiciousnessColor".
				iconMorph := methodMorph setColorIcon: suspiciousnessColor.
				node hasViolations ifTrue: [
					iconMorph addMorph: (SPMViolationMorph on: node violations) ] ] ].! !

!SDebuggerCallTreeModel methodsFor: 'printing' stamp: 'mp 10/31/2010 13:28:51.743'!
printInfosForNode: aCallNode
	| suspiciousMethod |
	suspiciousMethod := self getSuspiciousnessAndConfidenceFor: aCallNode.
	suspiciousMethod ifNil: [^ ''].
	^  '{', (suspiciousMethod value first * 100) asInteger asString, '%} '! !

!SDebuggerCallTreeModel methodsFor: 'searching' stamp: 'mp 10/31/2010 12:56:29.143'!
searchNode: aText
	| query |
	"Allows search queries such as {> 90 meaning all suspicious methods 
	with a value larger than 90 %"
	query := aText asString splitInteger.
	query first withBlanksTrimmed caseOf: {
	['{>'] -> [^ self searchSuspiciousNodes: query second operator: #>].
	['{<'] -> [^ self searchSuspiciousNodes: query second operator: #<].
	['{='] -> [^ self searchSuspiciousNodes: query second operator: #=].
	}
	otherwise: [^ super searchNode: aText].
	! !

!SDebuggerCallTreeModel methodsFor: 'searching' stamp: 'mp 7/16/2013 17:04:18.113'!
searchSuspiciousNodes: aNumber operator: aSymbol
	| foundedNodes nodeString |
	self clearHighlighting.
	self rootNode ifNotNil: [
		foundedNodes := OrderedCollection new.
		self traverseCallTree: self rootNode 
			with: [:aNode | 
				nodeString := aNode asString.
				((nodeString first = ${) and: 
					[nodeString splitInteger second perform: aSymbol with: aNumber])
						ifTrue: [foundedNodes add: aNode]].
		self 
			highlightedItems: foundedNodes;
			changed: #highlightItem:;
			selectNextSearchResult: foundedNodes.
		^ foundedNodes].! !

!SDebuggerCallTreeModel methodsFor: 'accessing' stamp: 'mp 3/25/2011 12:47:47.326'!
suspiciousMethods
	"Answer the value of suspiciousMethods"

	^ suspiciousMethods ifNil: [suspiciousMethods := 
								self pathFinder coverageTracer mostSuspiciousMethods]! !

!SDebuggerCallTreeModel methodsFor: 'accessing' stamp: 'mp 3/25/2011 12:25:31.106'!
suspiciousMethods: anObject
	"Set the value of suspiciousMethods"

	suspiciousMethods := anObject! !

MwMethodWrapper subclass: #SCallWrapper
	instanceVariableNames: 'ids tracer callsBefore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SCallWrapper commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SCallWrapper class methodsFor: 'instance creation' stamp: 'mp 1/25/2010 14:25'!
on: aSymbol inClass: aClass withTracer: aTracer

	| wrapper |
	wrapper := self on: aSymbol inClass: aClass.
	wrapper ifNotNil: [wrapper tracer: aTracer].
	^ wrapper! !

!SCallWrapper methodsFor: 'evaluating' stamp: 'mp 1/25/2010 18:10'!
afterMethod

	self tracer returnFromNode: self ids removeLast.! !

!SCallWrapper methodsFor: 'evaluating' stamp: 'mp 6/15/2010 16:09'!
beforeMethod: anObject
	self ids add: (self tracer 
				createNewCallNodeOn: self wrappedClass 
				with: self clientSelector
				receiverType: anObject class
				callsBefore: self callsBefore).
	self callsBefore: self callsBefore + 1.! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 6/15/2010 16:08'!
callsBefore
	"Answer the value of callsBefore"

	^ callsBefore! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 6/15/2010 16:08'!
callsBefore: anObject
	"Set the value of callsBefore"

	callsBefore := anObject! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 18:09'!
ids
	"Answer the value of ids"

	^ ids! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 18:09'!
ids: anObject
	"Set the value of ids"

	ids := anObject! !

!SCallWrapper methodsFor: '-- all --' stamp: 'mp 6/15/2010 16:09'!
initialize

	self ids: OrderedCollection new.
	self callsBefore: 0.! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 15:22'!
selectedView: aSymbol! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 14:24'!
tracer
	"Answer the value of tracer"

	^ tracer! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 14:24'!
tracer: anObject
	"Set the value of tracer"

	tracer := anObject! !

!SCallWrapper methodsFor: 'evaluating' stamp: 'mp 5/5/2010 12:18'!
valueWithReceiver: anObject arguments: anArrayOfObjects

	self 
		beforeMethod;
		beforeMethod: anObject.
	^ [
		self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects.
	] ensure: [self afterMethod]! !

SCallWrapper subclass: #SViewWrapper
	instanceVariableNames: 'selectedView'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SViewWrapper commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

SViewWrapper subclass: #SProfilingWrapper
	instanceVariableNames: 'stopwatches'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SProfilingWrapper commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SProfilingWrapper methodsFor: 'evaluating' stamp: 'mp 7/16/2013 11:39:40.559'!
afterMethod

	| stopwatch |
	stopwatch := self stopwatches removeLast.
	stopwatch suspend.
	self tracer addProfileInformation:  stopwatch duration node: self ids last.
	super afterMethod.! !

!SProfilingWrapper methodsFor: 'evaluating' stamp: 'mp 7/16/2013 11:39:46.494'!
beforeMethod

	super beforeMethod.
	self stopwatches add: Stopwatch new.
	self stopwatches last activate.! !

!SProfilingWrapper methodsFor: '-- all --' stamp: 'mp 7/16/2013 11:39:37.61'!
initialize

	super initialize.
	self stopwatches: OrderedCollection new.! !

!SProfilingWrapper methodsFor: 'accessing' stamp: 'mp 7/16/2013 11:39:29.995'!
stopwatches

	^ stopwatches! !

!SProfilingWrapper methodsFor: 'accessing' stamp: 'mp 7/16/2013 11:39:30'!
stopwatches: anObject

	stopwatches := anObject! !

!SViewWrapper methodsFor: 'accessing' stamp: 'mp 6/28/2010 22:28'!
getViewString: anObject

	self selectedView == #identities
		ifTrue: ["^ self wrappedClass asString, '(', anObject identityHash, ')'"
			^ anObject identityHash].		
	self selectedView == #objects
		ifTrue: [^ [anObject asString] ifError: ['Error cannot print object']].
	self selectedView == #classes
		ifTrue: [^ self wrappedClass asString].
! !

!SViewWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 14:38'!
selectedView
	"Answer the value of selectedView"

	^ selectedView! !

!SViewWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 14:38'!
selectedView: anObject
	"Set the value of selectedView"

	selectedView := anObject! !

!SViewWrapper methodsFor: 'evaluating' stamp: 'mp 5/5/2010 12:19'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	| viewString |
	self tracer isActive
		ifTrue: [
			self 
				beforeMethod;
				beforeMethod: anObject.
			self tracer stopTracing.
			viewString := self getViewString: anObject.
			self tracer addView: viewString to: self selectedView.
			self tracer startTracing.
			[^ self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects] 
				ensure: [self afterMethod]]
		ifFalse: [^ self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects].


! !

MwMethodWrapper subclass: #SMethodByteCodeCoverageForAllTestsWrapper
	instanceVariableNames: 'currentTest coveredByteCodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Debugger'!

!SMethodByteCodeCoverageForAllTestsWrapper commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SMethodByteCodeCoverageForAllTestsWrapper class methodsFor: 'instance creation' stamp: 'mp 11/5/2010 11:39:17.439'!
on: aMethodReference
	
	^ self on: aMethodReference methodSymbol inClass: aMethodReference actualClass! !

!SMethodByteCodeCoverageForAllTestsWrapper methodsFor: 'accessing' stamp: 'mp 11/5/2010 14:23:30.532'!
coveredByteCodes
	"Answer the value of coveredByteCodes"

	^ coveredByteCodes ifNil: [coveredByteCodes := Dictionary new]! !

!SMethodByteCodeCoverageForAllTestsWrapper methodsFor: 'accessing' stamp: 'mp 11/5/2010 14:23:08.362'!
coveredByteCodes: anObject
	"Set the value of coveredByteCodes"

	coveredByteCodes := anObject! !

!SMethodByteCodeCoverageForAllTestsWrapper methodsFor: 'accessing' stamp: 'mp 11/5/2010 14:23:08.35'!
currentTest
	"Answer the value of currentTest"

	^ currentTest! !

!SMethodByteCodeCoverageForAllTestsWrapper methodsFor: 'accessing' stamp: 'mp 11/5/2010 14:23:08.354'!
currentTest: anObject
	"Set the value of currentTest"

	currentTest := anObject! !

!SMethodByteCodeCoverageForAllTestsWrapper methodsFor: 'evaluating' stamp: 'mp 7/16/2013 11:35:51.151'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	| result byteCodes |
	result := thisContext
		runSimulatedMethodOnly: [self clientMethod 
										valueWithReceiver: anObject 
										arguments: anArrayOfObjects]
		contextAtEachStep: [ :context | 
			self compiledMethod == context method
				ifTrue: [	byteCodes := self coveredByteCodes 
											at: self currentTest ifAbsentPut: [Set new].
						byteCodes add: context pc]].
	^ result! !

MwMethodWrapper subclass: #SSpecificWrapper
	instanceVariableNames: 'numberOfCallsBefore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SSpecificWrapper commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

SSpecificWrapper subclass: #SByteCodeCoverageWrapper
	instanceVariableNames: 'coveredByteCodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SByteCodeCoverageWrapper commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SByteCodeCoverageWrapper methodsFor: 'accessing' stamp: 'mp 11/8/2010 15:21:28.79'!
coveredByteCodes
	"Answer the value of coveredByteCodes"

	^ coveredByteCodes ifNil: [coveredByteCodes := OrderedCollection new]! !

!SByteCodeCoverageWrapper methodsFor: 'accessing' stamp: 'mp 11/8/2010 15:19:07.504'!
coveredByteCodes: anObject
	"Set the value of coveredByteCodes"

	coveredByteCodes := anObject! !

!SByteCodeCoverageWrapper methodsFor: 'evaluating' stamp: 'mp 7/16/2013 11:38:21.509'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	| result |
	self isSpecificCall 
		ifTrue: [self decrementCallCount.
				result := thisContext
							runSimulatedMethodOnly: [self clientMethod 
															valueWithReceiver: anObject 
															arguments: anArrayOfObjects]
							contextAtEachStep: [ :context | 
								self compiledMethod == context method
									ifTrue: [	self coveredByteCodes add: context pc]]]
		ifFalse: [self decrementCallCount.
				result := self clientMethod 
								valueWithReceiver: anObject 
								arguments: anArrayOfObjects].
	^ result! !

SSpecificWrapper subclass: #SDebugWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SDebugWrapper commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SDebugWrapper methodsFor: 'as yet unclassified' stamp: 'mp 7/7/2010 17:00:04.386'!
beforeMethod

	self numberOfCallsBefore = 1
		ifTrue: [SBreakpointManager installInClass: self wrappedClass selector: self clientSelector.].
	self decrementCallCount.! !

SSpecificWrapper subclass: #SExploreWrapper
	instanceVariableNames: 'objects exploreWhat found'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SExploreWrapper commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SExploreWrapper class methodsFor: 'instance creation' stamp: 'mp 1/25/2010 17:37'!
on: aMethodSelector inClass: aClass waitingCalls: aNumber exploreWhat: aSymbol

	| wrapper |
	wrapper := self on: aMethodSelector inClass: aClass waitingCalls: aNumber.
	wrapper ifNotNil: [wrapper exploreWhat: aSymbol].
	^ wrapper! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 17:30'!
exploreWhat
	"Answer the value of exploreWhat"

	^ exploreWhat! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 17:30'!
exploreWhat: anObject
	"Set the value of exploreWhat"

	exploreWhat := anObject! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 18:48'!
found
	"Answer the value of found"

	^ found ifNil: [found := false]! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 18:48'!
found: anObject
	"Set the value of found"

	found := anObject! !

!SExploreWrapper methodsFor: 'exploring' stamp: 'mp 1/25/2010 17:34'!
getArguments: anArrayOfObjects

	(self isSpecificCall and: [self exploreWhat == #arguments])
		ifTrue: [self objects add: anArrayOfObjects veryDeepCopy].! !

!SExploreWrapper methodsFor: 'exploring' stamp: 'mp 3/5/2010 16:51'!
getReturn: anObject

	(self isSpecificCall and: [self exploreWhat == #return])
		ifTrue: [self objects add: anObject veryDeepCopy].! !

!SExploreWrapper methodsFor: 'exploring' stamp: 'bs 7/17/2010 14:32:28.652'!
getSelfAfter: anObject
	(self isSpecificCall and: [self exploreWhat == #selfAfter])
		ifTrue: [self objects add: anObject veryDeepCopy].! !

!SExploreWrapper methodsFor: 'exploring' stamp: 'mp 1/25/2010 17:35'!
getSelfBefore: anObject

	(self isSpecificCall and: [self exploreWhat == #selfBefore])
		ifTrue: [self objects add: anObject veryDeepCopy].! !

!SExploreWrapper methodsFor: 'exploring' stamp: 'mp 3/4/2010 09:50'!
getTypeInformation: anArrayOfObjects of: aSymbol

	| type |
	((self isSpecificCall and: [self exploreWhat == #types]) and: [anArrayOfObjects isEmpty not])
		ifTrue: [	anArrayOfObjects size = 1
					ifTrue: [type := anArrayOfObjects first class]
					ifFalse: [type := OrderedCollection new.
							anArrayOfObjects do: [:each | type add: each class]].
				self objects add: aSymbol -> type].! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 17:30'!
objects
	"Answer the value of objects"

	^ objects ifNil: [objects := OrderedCollection new]! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 17:30'!
objects: anObject
	"Set the value of objects"

	objects := anObject! !

!SExploreWrapper methodsFor: '-- all --' stamp: 'bs 7/17/2010 16:00:15.791'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	| result |
	"#arguments #selfBefore #selfAfter #return #types"	
	self 
		getTypeInformation: anArrayOfObjects of: #arguments;
		getTypeInformation: (Array with: anObject) of: #self;
		getArguments: anArrayOfObjects;
		getSelfBefore: anObject.
	self isSpecificCall ifTrue: [self found: true].
	self decrementCallCount.
	^ [
		result := self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects.
		result.
	] ensure: [ self found 
				ifTrue: [self incrementCallCount.
					self 
						getSelfAfter: anObject;
						getReturn: result;
						getTypeInformation: (Array with: result) of: #result.
					self isSpecificCall 
						ifTrue: [self found: false. 
							      self decrementCallCount]]]
! !

!SSpecificWrapper class methodsFor: 'instance creation' stamp: 'mp 1/25/2010 16:40'!
on: aSymbol inClass: aClass waitingCalls: aNumber

	| wrapper |
	wrapper := self on: aSymbol inClass: aClass.
	wrapper ifNotNil: [wrapper numberOfCallsBefore: aNumber].
	^ wrapper! !

!SSpecificWrapper methodsFor: 'count' stamp: 'mp 1/25/2010 16:39'!
decrementCallCount
	
	self numberOfCallsBefore: self numberOfCallsBefore - 1.! !

!SSpecificWrapper methodsFor: 'count' stamp: 'mp 1/25/2010 19:07'!
incrementCallCount
	
	self numberOfCallsBefore: self numberOfCallsBefore + 1.! !

!SSpecificWrapper methodsFor: 'count' stamp: 'mp 1/25/2010 16:40'!
isSpecificCall
	
	^ self numberOfCallsBefore = 0! !

!SSpecificWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 16:39'!
numberOfCallsBefore
	"Answer the value of numberOfCallsBefore"

	^ numberOfCallsBefore! !

!SSpecificWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 16:39'!
numberOfCallsBefore: anObject
	"Set the value of numberOfCallsBefore"

	numberOfCallsBefore := anObject! !

SCodeExplorerIndentingListItemMorph subclass: #SDebuggerCodeExplorerIndentingListItemMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Debugger'!

!SDebuggerCodeExplorerIndentingListItemMorph commentStamp: '<historical>' prior: 0!
The Path Tools Framework is released under the MIT License (MIT).

Copyright (c) 2009 Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!SDebuggerCodeExplorerIndentingListItemMorph methodsFor: 'accessing' stamp: 'mp 11/8/2010 16:32:22.369'!
codePane
	"Answer the value of codePane"
	| suspiciousColor |
	^ codePane ifNil: [
		self icon ifNotNil: [suspiciousColor := self icon colorAt: 1@1].
		codePane := SDebuggerCodePaneMorph 
										on: self callNode 
										color: self getBackgroundColor
										suspiciousColor: suspiciousColor].! !
